package com.zhentao.config;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.zhentao.filter.JwtFilter;
import com.zhentao.pojo.Permission;
import com.zhentao.pojo.User;
import com.zhentao.vo.Result;
import com.zhentao.filter.JwtFilter;
import com.zhentao.mapper.PermissionMapper;
import com.zhentao.utils.CacheService;
import com.zhentao.utils.JwtService;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.authorization.AuthorizationDecision;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.access.intercept.RequestAuthorizationContext;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.TimeUnit;
import java.util.function.Supplier;

@Configuration
@EnableWebSecurity
public class SecurityConfig {
    @Autowired
    private JwtFilter jwtFilter;
    @Autowired
    private CacheService cacheService;
    @Autowired
    private PermissionMapper permissionMapper;
    @Bean
    public SecurityFilterChain securityFilterChain(HttpSecurity http) throws Exception {
        //关闭csrf
        http.csrf(config->config.disable());
        http.authorizeHttpRequests(config->config.requestMatchers("/user/login1","/user/login","/user/yan","/user/test").permitAll().anyRequest().access(this::check));
        http.formLogin(config->config.loginProcessingUrl("/user/login").successHandler(this::loginsuccess).failureHandler(this::loginshibai));
        http.logout(config->config.logoutUrl("/user/logout").logoutSuccessHandler(this::logout));
        http.exceptionHandling(config->config.accessDeniedHandler(this::meideng).authenticationEntryPoint(this::meiquan));
        //关闭session缓存
        http.sessionManagement(config->config.sessionCreationPolicy(SessionCreationPolicy.STATELESS));
        http.addFilterBefore(jwtFilter, UsernamePasswordAuthenticationFilter.class);
//        http.userDetailsService(userDetailsService);
        return http.build();
    }

    //正确退出
    private void logout(HttpServletRequest request, HttpServletResponse response, Authentication authentication) throws IOException {
        Result result = Result.OK("登出成功",null);
        response.setContentType("application/json;charset=utf-8");
        response.getWriter().write(JSON.toJSONString(result));
    }
    //没有登陆处理
    private void meideng(HttpServletRequest request, HttpServletResponse response, AccessDeniedException accessDeniedException) throws IOException {
        Result result = Result.ERR("没权限",null);
        response.setContentType("application/json;charset=utf-8");
        response.getWriter().write(JSON.toJSONString(result));
    }
    //登录失败处理

    private void loginshibai(HttpServletRequest request, HttpServletResponse response, AuthenticationException exception) throws IOException {
        Result result = Result.ERR("登录失败",null);
        response.setContentType("application/json;charset=utf-8");
        response.getWriter().write(JSON.toJSONString(result));
    }

    //没有权限处理
    private void meiquan(HttpServletRequest request, HttpServletResponse response, AuthenticationException exception) throws IOException {
        Result result = Result.ERR("没登录",null);
        response.setContentType("application/json;charset=utf-8");
        response.getWriter().write(JSON.toJSONString(result));
    }

    //校验处理
    private AuthorizationDecision check(Supplier<Authentication> authenticationSupplier, RequestAuthorizationContext context) {
        Object principal = authenticationSupplier.get().getPrincipal();
        if(principal instanceof String){
            return new AuthorizationDecision(false);
        }
        String uri = context.getRequest().getRequestURI();
        System.out.println(uri);
        System.out.println("11111111111111");
        QueryWrapper<Permission> wrapper = new QueryWrapper<>();
        wrapper.eq("path",uri);
        Permission permission = permissionMapper.selectOne(wrapper);
        System.err.println(permission);
        if(permission==null){
            return new AuthorizationDecision(false);
        }
        User user = (User) principal;
        List<String> perms = user.getPerms();
        String perms1 = permission.getPerms();
        //查询是否有权限
        System.err.println(perms1);
        if(perms.contains(perms1)){
            return new AuthorizationDecision(true);
        }
        return new AuthorizationDecision(false);
    }

    private void loginsuccess(HttpServletRequest request, HttpServletResponse response, Authentication authentication) throws IOException {
        User user = (User) authentication.getPrincipal();
        Map<String,Object> map = new HashMap<>();
        map.put("id",user.getId());
        map.put("username",user.getUsername());
        map.put("phone",user.getPhone());
        map.put("perms",user.getPerms());
        String token = JwtService.createToken(map);
        System.err.println(token);
        String uuid = UUID.randomUUID().toString();
        String key = "aaa" + uuid;
        cacheService.setEx(key,token,10, TimeUnit.DAYS);
        map.put("uuid",uuid);
        map.put("token",token);
        Result result = Result.OK("登陆成功",map);
        response.setContentType("application/json;charset=utf-8");
        response.getWriter().write(JSON.toJSONString(result));
    }

    @Bean
    public BCryptPasswordEncoder bCryptPasswordEncoder(){
        return new BCryptPasswordEncoder();
    }
}
